#export PYTHONPATH=/home/acue/data/rd/p-open-now_lap001/filesysobjects/componens/pyfilesysobjects:$PYTHONPATH
#export PYTHONPATH=/home/acue/data/rd/p-open-now_lap001/sourceinfo/components/pysourceinfo:$PYTHONPATH
#export PYTHONPATH=/home/acue/data/rd/p-open-now_lap001/subprocs/components/pysubprocs:$PYTHONPATH
export PYTHONPATH=$PWD:$PYTHONPATH
export PATH=$PWD/bin:$PATH

